/*
 * Copyright (c) 2007-2010 Koji Suzuki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))

#define nop()	__asm__ volatile ("nop" )
#define sleep()	__asm__ volatile ("sleep" )

#ifndef TIFR0
#define TIFR0	TIFR
#endif


#ifdef HAVE_CONFIH_H
#include "config.h"
#else
#define BUTTON_SUPPORT
// backlight : PD5/OC0B 
#define BACKLIGHT_PORT	PORTD
#define BACKLIGHT_DDR	DDRD
#define BACKLIGHT	PD5

// V0 : PB4/OC1B 
#define V0_PORT	PORTB
#define V0_DDR	DDRB
#define V0		PB4

#endif

#include "button.h"
#include "i2cdev.h"
#include "lcd.h"

uint8_t button_reg;
uint8_t extmode; // change with Set CG Addr. 
uint8_t contrast;
uint8_t backlight;

static void set_contrast(uint8_t c) {
    if (!c) {
        TCCR1A = 0;
    } else {
        OCR1B = (uint16_t)c<<6; // max 1/4 VCC
        TCCR1A = 2<<COM1B0;
    }
}

static void set_backlight(uint8_t b) {
    if (!b) {
        TCCR0A = 0;
    } else {
        OCR0B = b;
        TCCR0A = 2<<COM0B0;
    }
}

int main(void)
{
    int8_t r;
    uint8_t d;
    uint8_t rs = 0;
    uint8_t cont = 0;

    /* set system_clock pre scaler 1/8 => 1/1 */
    CLKPR = (1<<CLKPCE);
    CLKPR = 0;

    lcd_init();
    i2cdev_init(0x3e);
    //usart_init();
    //init_i2c();

    bit_set(BACKLIGHT_DDR, BACKLIGHT);
    bit_clear(BACKLIGHT_PORT, BACKLIGHT);
    bit_set(V0_DDR, V0);
    bit_clear(V0_PORT, V0);
    TCCR0B = 0x4; // 1/256 free running (122 Hz)
    TCCR1B = 0x1; // 1/1 free running (122 Hz)
#ifdef BUTTON_SUPPORT
    i2cdev_set_rx_buf(&button_reg, 1);
#endif
    for(;;) {

	r = i2cdev_can_getc();
	if (r <0) {
	    i2cdev_flush();
	    cont = 0;
	} else if (r) {
	    if (!cont) {
	        d = i2cdev_getc();
	        rs = d & 0x40;
		cont = ((d & 0x80) == 0);
	    }
	    if (i2cdev_can_getc() > 0) {
		d = i2cdev_getc();
		if (rs) lcd_data(d);
		else {
		    if ((d & 0xe0) == 0x20) {	// Function Set
			d &= ~0x10;		// 4 bit-mode only
			extmode = d & 0x01;
			lcd_cmd(d);
		    } else if (extmode) {
			switch (d >> 4) {
			case 0x4:
				backlight = (d << 4);
				set_backlight(backlight);
				break;
			case 0x5:
				contrast = (d << 4);
				set_contrast(contrast);
				break;
			case 0x6:
				backlight &= 0xf0;
				backlight |= (d & 0xf);
				set_backlight(backlight);
				break;
			case 0x7:
				contrast &= 0xf0;
				contrast |= (d & 0xf);
				set_contrast(contrast);
				break;
			default:
				lcd_cmd(d);
			}
		    } else {
			lcd_cmd(d);
		    }
		}
	    }
	}
#ifdef BUTTON_SUPPORT
	if (bit_is_set(TIFR0, TOV0)) {
		button_reg = check_button() & 0x7;
		bit_set(TIFR0, TOV0);
	}
#endif
    }
    return 0;
}

/* circuit memo


ATtiny2313     LCD interface

                 DB0-DB3 (NC)
  PB0  ---------- DB4
  PB1  ---------- DB5
  PB2  ---------- DB6
  PB3  ---------- DB7 --- 22k -- VCC
              
  PB6 ----------- E
  PD6 ----------- RS

  PB4 --10k---+-- V0 
              |
              C 10u
              |
             GND

                +5V
                 |
                BLA
                
                BLK
                 |
                (R) 
                 |
                 C 
  PD5 --- R -- B      NPN Tr
                 E
                 |
                GND

ATtiny2313     I2C slave interface

 PB7 ------------ SCL
 PB5 ------------ SDA 

ATtiny2313     BUTTON interface

                                  PB0 PB1 PB2
                              _    |   |   |
  PD3 -+-------|>|---3.3K ---o o---+---|---|---- 33k --- GND
       |                               |   |
       |                      _        |   |
       +-------|>|---3.3K ---o o-------+---|---- 33k --- GND
       |                                   |
       |                      _            |
       +-------|>|---3.3K ---o o-----------+---- 33k --- GND



Command:

*/
